
# This file is automatically generated by setup.py, So don't edit it. :)
VERSION_INFO = {
    'final': True,
    'version': '1.8.5.post1',
    'branch_nick': '(detached',
    'revision_id': '1eb4fdfe55067f260f792778161b9ad4ce045c21',
    'revno': 4507
}
